@@ -53,6 +53,12 @@ def lt(a, b):
 def gt(a, b):
     assert a > b, "%r <= %r" % (a, b)
 
+def isin(a, b):
+    assert a in b, "%r is not in %r" % (a, b)
+
+def isnotin(a, b):
+    assert a not in b, "%r is in %r" % (a, b)
+
 def has(a, b):
     assert hasattr(a, b), "%r has no attribute %r" % (a, b)
 
@@ -65,6 +71,7 @@ def raises(exctype, func, *args, **kwargs):
     except exctype, inst:
         pass
     else:
+        func_name = getattr(func, "func_name", "<builtin_function>")
         raise AssertionError("Function %s did not raise %s" % (
-            func.func_name, exctype.__name__))
+            func_name, exctype.__name__))
 
